import java.util.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import org.json.*;

public class Meeting {

	private static ArrayList<User> table_User;
	private static SimpleDateFormat parseDate = new SimpleDateFormat("dd/MM/yyyy");
	private static Output output = new Output();
	private static Input input = new Input(output);
	private static Check check = new Check(output);
	private static Request req = new Request(check);

	public static void main(String[] args) throws Exception {

		// if not connected to internet => exit program
		check.check_connection();

		// Create and set the users table
		table_User = new ArrayList<User>();
		set_t_user();

		ArrayList<MyDate> calendar = get_calendar();
		print_calendar(calendar);

		output.section("Voici les utilisateurs entrer:");
		table_User.forEach((u) -> print_user(u));

		output.section("Préparation des points géographiques...");
		ArrayList<Location> t_loc = new ArrayList<Location>();
		table_User.forEach((u) -> t_loc.add(u.get_location()));
		output.success();

		City city = new City(t_loc, output);

		output.section("Calcul du centre géogrpahique...");
		city.calcul_geo_center();
		output.print("Centre x: " + city.get_geo_center().get_x() + ", y: " + city.get_geo_center().get_y());
		output.success();

		output.section("Correction du centre par rapport aux poids...");
		output.success();

		/*
		* output.section("Paramètres pour le choix de la ville:");
		* output.print("Besoin d'une bibliothèque ? (y/n)");
		* if(input.nextLine().equals("y")) city.set_lib(true);
		* output.print("Besoin d'une université ? (y/n)");
		* if(input.nextLine().equals("y")) city.set_uni(true); output.success();
		*/

		output.section("Choix de la ville...");
		city.set_city(get_city_nearby(city.get_geo_center()));
		output.success("La ville du rdv est: " + city.get_city());

		Appointment appointment = new Appointment(calendar, check);

		output.section("Calcul de la date et heure idéal...");
		appointment.most_popular_date();
		output.print("Rendez-vous fixé le: " + parseDate.format(appointment.get_f_date().get_date()) + " de "
									+ appointment.get_f_date().get_t_from().to_string() + " à "
									+ appointment.get_f_date().get_t_to().to_string());
		output.success();

		output.section("Validation des données...");
		output.success();

		output.section("Calcul des chemins...");
		set_paths(city.get_city(), appointment.get_f_date());
		output.success();

		output.section("Rendus des résultats...");
		output.printN("Le rendez-vous aura lieu le: " + parseDate.format(appointment.get_f_date().get_date()) + " de "
									+ appointment.get_f_date().get_t_from().to_string() + " à "
									+ appointment.get_f_date().get_t_to().to_string());
		for (User u : table_User) {
			try{
				u.print_journey();
			}catch(Exception ex){
				output.print("Quelle chance vous habitez sur place !");
			}
		}
		output.success();
	}

	/**
	* Print all user data
	*
	* @param user [description]
	*/
	private static void print_user(User user) {
		output.section("Utilisateurs:");
		output.print("Coordonées x, y: " + user.get_location().get_x() + ", " + user.get_location().get_y());
		output.print("Date possible :");
		user.get_calendar().forEach((d) -> System.out.println(parseDate.format(d.get_date())));
		System.out.println();
	}

	/**
	* Print all date from a dates table
	*
	* @param cal ArrayList<MyDate>, array of date
	*/
	private static void print_calendar(ArrayList<MyDate> cal) {
		output.section("General calendar:");
		cal.forEach((d) -> System.out.println(parseDate.format(d.get_date())));
		output.success();
	}

	/**
	* Start to calculate each user's journey
	*
	* @param dest, String of the destination city
	* @throws Exception
	*/
	private static void set_paths(String dest, MyDate date) throws Exception {
		for (User u : table_User) {
			if(!u.get_location().get_place().equals(dest)) {
				u.calcul_path(dest, date);
			}
		}
	}

	/**
	* Set all users with: their location, their calendar
	*/
	private static void set_t_user() throws Exception {

		boolean end = false;
		while (!end) {
			output.section("Bonjour, veuillez entrer les informations utilisateur:");

			double x = 0;
			double y = 0;
			String town = "";

			int choice = 0;

			boolean e = false;
			while (!e) {
				output.print("Votre ville:");
				input.nextLine();

				// Location loc = req.get_location_from_city(input.get_value());
				String result = req.get_location_from_city(input.get_value());
				JSONArray r = new JSONArray(result);
				output.section("Voici les résultats de votre recherche (" + r.length() + " résultats) :");
				for (int i = 0; i < r.length(); i++) {
					output.print(i + ")   " + r.getJSONObject(i).getString("display_name"));
				}
				output.printN("Aucun > Appuyez sur n'importe quelle touche!");

				output.print("\nVeuillez choisir le numéro correspondant:");
				input.nextLine();

				try {
					choice = Integer.parseInt(input.get_value());

					if (choice < r.length()) {
						e = true;
					}

					x = Double.parseDouble(r.getJSONObject(choice).getString("lon"));
					y = Double.parseDouble(r.getJSONObject(choice).getString("lat"));
					town = r.getJSONObject(choice).getJSONObject("address").getString("city").toString();

				} catch (Exception error) {
					output.error("Veuillez entrer un nombre valide !");
				}


			}

			output.success("Vous avez choisi : " + town + " / coord: x: " + x + ", y: " + y);

			Location loc = new Location(x, y, town);
			ArrayList<MyDate> t_date = set_date();

			table_User.add(new User(loc, t_date, output, req));

			if (table_User.size() >= 2) {
				output.section("Avez-vous fini d'entrer les utilisateurs ? (y/n)");
				if (input.nextLine().equals("y"))
				end = true;
			}
		}
	}

	/**
	* Set table of date's user
	* @return ArrayList of date
	*/
	private static ArrayList<MyDate> set_date() throws Exception {
		ArrayList<MyDate> t_date = new ArrayList<MyDate>();

		boolean end = false;
		while (!end) {
			output.section("Date possible (jour/mois/année heure:min heure:min):");
			String[] line = input.nextLine().split(" ");

			// check the validity of all MyDate arguments
			if (check.myDate_validity(line)) {
				String[] from = line[1].split(":");
				String[] to = line[2].split(":");

				output.success("Vous avez saisi : " + line[0] + " de " + line[1] + " à " + line[2]);

				MyDate d = new MyDate(parseDate.parse(line[0]),
				new Time(Integer.parseInt(from[0]), Integer.parseInt(from[1])),
				new Time(Integer.parseInt(to[0]), Integer.parseInt(to[1])));

				t_date.add(d);

				output.section("Avez-vous entrer toutes les t_date ? (y/n)");
				if (input.nextLine().equals("y"))
				end = true;
			}
		}

		return t_date;
	}

	/**
	* Concat all user's calendar
	* @return ArrayList<MyDate>, calendar of date
	*/
	private static ArrayList<MyDate> get_calendar() {
		ArrayList<MyDate> calendar = new ArrayList<MyDate>();

		for (User u : table_User)
		calendar.addAll(u.get_calendar()); // concat ArrayList

		return calendar;
	}

	/**
	 * Get cities nearby geographic location
	 * @param  loc       [description]
	 * @return           [description]
	 * @throws Exception [description]
	 */
	private static String get_city_nearby(Location loc) throws Exception {
		int choice = 0;
		JSONArray r = new JSONArray();
		boolean e = false;
		while(!e){
			String result = req.get_city(loc);
			r = new JSONObject(result).getJSONArray("data");

			output.section("Voici les villes proposées (" + r.length() + " résultats) :");

			for (int i = 0; i < r.length(); i++) {
				if(r.getJSONObject(i).getString("country").equals("Suisse")){
					output.print(i + ")   " + r.getJSONObject(i).getString("city") + ", " + r.getJSONObject(i).getString("country"));
				}
			}

			output.printN("Aucun > Appuyez sur n'importe quelle touche!");

			output.print("\nVeuillez choisir le numéro correspondant:");
			input.nextLine();

			try {
				choice = Integer.parseInt(input.get_value());

				if (choice < r.length()) {
					e = true;

				}
				//output.success("Voici t'as ville frère: " + r.getJSONObject(choice).getString("city"));

			} catch (Exception error) {
				output.error("Veuillez entrer un nombre valide !");
			}
		}

		return r.getJSONObject(choice).getString("city").toString();
	}
}
