import java.util.ArrayList;
import java.util.regex.Pattern;
import java.util.regex.*;

import org.json.*;

public class Path {

  private String from;
  private String to;
  private String duration;

  private JSONObject path;

  private ArrayList<Step> steps;
  private static Output output;

  public Path(JSONObject path, Output out) throws Exception{
    this.output = out;
	  this.from = path.getJSONObject("from").getJSONObject("station").getString("name");
	  this.to = path.getJSONObject("to").getJSONObject("station").getString("name");
	  this.duration = this.cut_duration(path.getString("duration"));

	  this.path = path;
	  this.steps = new ArrayList<Step>();

	  this.set_steps();
  }

  /**
   *
   * @throws Exception
   */
  private void set_steps() throws Exception{
	  JSONArray sections = this.path.getJSONArray("sections");
	  for(int i = 0; i < sections.length(); i++) {

		  JSONObject journey = sections.getJSONObject(i);

      String transport = "A pied";

      String platform_d = "";
		  String platform_a = "";
		  try{
		      transport = this.get_transport(journey.getJSONObject("journey"));

          platform_d = journey.getJSONObject("departure").getString("platform");
    		  platform_a = journey.getJSONObject("arrival").getString("platform");
      }catch(Exception e){
      }

      String from = journey.getJSONObject("departure").getJSONObject("station").getString("name");
      String to = journey.getJSONObject("arrival").getJSONObject("station").getString("name");

      String time_d = journey.getJSONObject("departure").getString("departure");
      String time_a = journey.getJSONObject("arrival").getString("arrival");

		  time_d = this.cut_time(time_d);
		  time_a = this.cut_time(time_a);

		  this.steps.add(new Step(from, to, transport, time_d, time_a, platform_d, platform_a));
	  }
  }

  /**
   *
   * @param journey
   * @return
   * @throws Exception
   */
  private String get_transport(JSONObject journey) throws Exception{
	  String r;
	  r = journey.getString("category")+ " " + journey.getString("number");
	  return r;
  }

  private String cut_duration(String dur) {
	  String[] r = dur.split("d");
	  r = r[1].split(":");
	  return new String(r[0] + ":" + r[1]);
  }

  /**
   *
   * @param str
   * @return
   */
  private String cut_time(String str) {
	  String[] r = str.split("T");
	  r = r[1].split(":");
	  return new String(r[0] + ":" + r[1]);
  }

  /**
   *
   */
  public void print_journey() {

	this.output.section("Parcours de " + this.from + " à " + this.to);
	this.output.printN("Durée du trajet: " + this.duration);
	this.output.printN("Heure                Voyage                Platforme");
	for(Step s: this.steps) {
		s.print_step();
		this.output.print("----------------------------------------------------");
	}
  }

}

class Step{

	private String from;
	private String to;

	private String transport;
	private String platform_d; // platform departure
	private String platform_a; // platform arrival


	private String time_d; // time departure
	private String time_a; // time arrival

	public Step(String from, String to, String transport, String t_d, String t_a, String p_d, String p_a){
		this.from = from;
		this.to = to;

		this.transport = transport;
		this.platform_d = p_d;
		this.platform_a = p_a;

		this.time_d = t_d;
		this.time_a = t_a;
	}


	public void print_step() {
		Output out = new Output();
		out.print(this.time_d + "                " + this.from + "                " + this.platform_d);
		out.print("                     " + this.transport);
		out.print(this.time_a + "                " + this.to + "                " + this.platform_a);


	}
}
