import java.util.*;
import java.net.*;
import java.io.*;
import org.json.*;

public class User {

	private Location location; // location x, y
	private ArrayList<MyDate> calendar = new ArrayList<MyDate>();
	private Path path;
	private static Output output;
	private static Request req;

	/**
	 * User constructor
	 *
	 * @param x     float, coordinate x
	 * @param y     float, coordinate y
	 * @param dates ArrayList<MyDate>, table of possible dates
	 * @param out
	 */
	public User(float x, float y, ArrayList<MyDate> dates, Output out, Request r) {
		this.output = out;
		this.req = r;

		this.location = new Location(x, y);
		this.calendar = dates;
	}
	/**
	 * User constructor
	 *
	 * @param location [description]
	 * @param dates    [description]
	 * @param out      [description]
	 */
	public User(Location location, ArrayList<MyDate> dates, Output out, Request r) {
		this.output = out;
		this.req = r;

		this.location = location;
		this.calendar = dates;
	}

	/**
	 * Get coordinate of the user
	 *
	 * @return Location
	 */
	public Location get_location() {
		return this.location;
	}

	/**
	 * Get all possible dates from the user
	 *
	 * @return ArrayList<MyDate>, table of dates
	 */
	public ArrayList<MyDate> get_calendar() {
		return this.calendar;
	}

	/**
	 * Set locaton coordinate x and y
	 *
	 * @param x Float, Coordinate x
	 * @param y Float, Coordinate y
	 */
	public void set_location(float x, float y) {
		this.location.set_x(x);
		this.location.set_y(y);
	}

	/**
	 * Add a Date in the calendar
	 *
	 * @param date int[]
	 * @param from int[]
	 * @param to   int[]
	 */
	public void add_Date(Date date, Time from, Time to) {
		MyDate d = new MyDate(date, from, to);
		this.calendar.add(d);
	}

	public void calcul_path(String dest, MyDate date) throws Exception {
		String result = this.req.get_path(this.location.get_place(), dest, date);

		JSONObject jsonObj = new JSONObject(result);

		JSONObject path = jsonObj.getJSONArray("connections").getJSONObject(0);

		this.path = new Path(path, this.output);
	}

	public void print_journey() {
		this.path.print_journey();
	}

}
