"use strict"
var request = require('request')
var config = require('../config/config.json')
var search = require('../class/appointement/search_cities')

module.exports = {
  get_city: (name) => {
    const url = "https://nominatim.openstreetmap.org/search?email=marc888@hotmail.ch&q=" + name + "&format=json&addressdetails=1"

    return new Promise(function(resolve, reject){

      request(url, function (error, response, body) {
        if(error){
          reject(error)
        }else{
          body = JSON.parse(body)

          console.log(body);

          console.log("Get city: " + "cholet" + " for entry: " + name);

          resolve("cholet")
        }
      })

    })
  },

  get_cities: (point) => {
    var city_param = config.city_search

    var range = city_param.range
    var limit = city_param.limit
    var population = city_param.population
    var method = city_param.method

    switch (method) {
      case "geodb":
      return geodb(point, limit, population, range)
      break;

      case "local":
      return search.load_cities(point.lat, point.lon, range, population, limit)
      break;

    }

  },

  get_path_duration: (from, to, date) => {
    return new Promise(function(resolve){
      var path = transport_api(from, to, date)

      path.then(function(res){
        resolve(res)
      })
    })

  },

  get_path: (from, to, date) => {
    var method = config.transport_search.method

    return new Promise(function(resolve){
      switch (method) {
        case "transport_api":

        var path = transport_api(from, to, date)

        path.then(function(res){
          //console.log(res);
          resolve(transport_api_template(res, method))
        })
        break;
      }

    })
  },

  get_coordinate: (address) => {
    var coordinate = {lat: 0, lon:0}

    const url = "https://nominatim.openstreetmap.org/search?email=marc888@hotmail.ch&q=" + address + "&format=json&addressdetails=1"

    return new Promise(function(resolve, reject){

      request(url, function (error, response, body) {
        if(error){
          reject(error)
        }else{
          body = JSON.parse(body)

          coordinate.lat = parseFloat(body[0].lat)
          coordinate.lon = parseFloat(body[0].lon)

          console.log("Adding coordinate: " + coordinate.lat + ", " + coordinate.lon + " for address: " + address);

          resolve(coordinate)
        }
      })

    })
  }
}



function transport_api_template(path, method){
  var data = {
    time:{
      hour: 0,
      minutes: 0
    },
    steps:[]
  }

  switch (method) {
    case "transport_api":

    if (path.connections.length > 0) {

      var t = path.connections[0].duration.split('d')[1].split(':')

      data.time.hour = t[0]
      data.time.minutes = t[1]

      for (var i = 0; i < path.connections[0].sections.length; i++) {
        var step = path.connections[0].sections[i]


        var t1 = step.departure.departure.split('T')[1].split(':')
        var t2 = step.arrival.arrival.split('T')[1].split(':')

        data.steps.push({
          transport: step.journey.name,
          from:{
            station: step.departure.station.name,
            platform: step.departure.platform,
            hour: t1[0],
            minutes: t1[1]
          },
          to:{
            station: step.arrival.station.name,
            platform: step.arrival.platform,
            hour: t2[0],
            minutes: t2[1]
          }
        })

      }

    }

    return data

    break;
  }

}

function geodb(point, limit, population, range){
  limit = limit > 10 ? 10 : limit
  var lang = "fr"

  const url = "http://geodb-free-service.wirefreethought.com/v1/geo/locations/" + point.lat + "+" + point.lon + "/nearbyCities?limit=" + limit +"&offset=0&minPopulation=" + population + "&radius=" + range + "&languageCode=" + lang + "&sort=-population&types=CITY&countryIds=CH"

  return new Promise(function(resolve, reject){
    var cities = []

    request(url, function (error, response, body) {
      if(error){
        reject(error)
      }else{
        body = JSON.parse(body)['data'], cities = []

        /*for (var i = 0; i < body.length; i++) {

          if(body[i]['type'] == "CITY"){
            if(body[i]['countryCode'] == "CH"){
              cities.push(body[i]['city'])
            }
          }
        }*/
        var cities = []
        for (var i = 0; i < body.length; i++) {
              cities.push(body[i]['city'])
        }
        console.log(cities);
        resolve(cities)
      }
    })

  })
}

function transport_api(from, to, date){
  const url = "https://transport.opendata.ch/v1/connections?from=" + from + "&to=" + to + "&date="
  + date.day + "&time=" + date.time + "&isArrivalTime=1"

  return new Promise(function(resolve, reject){

    request(url, function (error, response, body) {
      if(error){
        reject(error)

      }else{
        //console.log(JSON.parse(body));
        resolve(JSON.parse(body))
      }

    })

  })
}
