"use strict"
var db = require('./connect_db')

module.exports = {

  // ----- GET -----
  get_meeting: (idMeeting, callback) => {
    let query = "SELECT * FROM t_meeting WHERE idMeeting = ?";

    db.query(query, idMeeting, (error, results, fields) => {
      if (error) throw error;

      callback(JSON.parse(results[0]['data']))
    })
  },

  get_user: (idMeeting, idUser, callback) => {
    get_users(idMeeting, function(users){
      callback(users[idUser])
    })
  },

  get_ids: (callback) => {
    let query = "SELECT idMeeting FROM t_meeting"

    db.query(query, (error, results, fields) => {
      if (error) throw error;

      callback(results)
    })
  },

  get_emails: (idMeeting, callback) => {
    get_users(idMeeting, function(users){
      var emails = []

      users.forEach(function(u){
        emails.push(u.email)
      })

      callback(emails)
    })
  },

  get_calendar: (idMeeting, callback) => {
    get_users(idMeeting, function(users){
      var calendar = []

      users.forEach(function(u){
        u.calendar.forEach(function(d){
          calendar.push(d)
        })
      })

      callback(users.length, calendar)
    })
  },

  get_locations: (idMeeting, callback) => {

    get_users(idMeeting, function(users){
      var tableLoc = new Array(users.length)

      for (var i = 0; i < users.length; i++) {
        tableLoc[i] = {lat: users[i].coordinate.lat, lon: users[i].coordinate.lon}
      }

      callback(tableLoc)
    })
  },
  get_cities: (idMeeting, callback) => {

    get_users(idMeeting, function(users){
      var tableCities = []

      users.forEach(function(u){
        tableCities.push(u.address)
      })

      callback(tableCities)
    })
  },

  get_times: (idMeeting, callback) => {

    get_users(idMeeting, function(users){
      var tableTimes = []

      for (var i = 0; i < users.length; i++) {
        tableTimes.push(parseInt(users[i].journey.time.hour) * 60 + parseInt(users[i].journey.time.minutes))
      }

      callback(tableTimes)
    })
  },




  // ----- ADD -----

  add_user: (idMeeting, data, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_ARRAY_APPEND(data, '$.user_table', CAST(? AS JSON)) WHERE idMeeting = ?"

    db.query(query, [data, idMeeting], (error, results, fields) => {
      if (error) throw error;
      callback(results)
    })
  },

  create_meeting: (id, data, callback) => {
    let query = "INSERT INTO t_meeting (idMeeting, data) VALUES (?, ?);"

    db.query(query, [id, data], (error, results, fields) => {
      if (error) throw error;
      console.log("Meeting created");

      callback(results)
    })
  },



  // ----- UPDATE -----
  update_user: (idMeeting, idUser, data, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_SET(data, '$.user_table[?]', CAST(? AS JSON)) WHERE idMeeting = ?"

    db.query(query, [idUser, data, idMeeting], (error, results, fields) => {
      if (error) throw error;

      console.log('User', idUser, 'have been updated:', data);

      callback(results)
    })
  },

  update_city_to: (idMeeting, city_to, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_SET(data, '$.location', ?) WHERE idMeeting = ?"


    db.query(query, [city_to, idMeeting], (error, results, fields) => {
      if (error) throw error;

      console.log('Appointement city changed to : ' + city_to);
      callback(results)
    })
  },

  update_appointement: (idMeeting, date, percentage, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_SET(data, '$.date', CAST(? AS JSON), '$.percentage', ?) WHERE idMeeting = ?"

    db.query(query, [date, percentage, idMeeting], (error, results, fields) => {
      if (error) throw error;

      console.log('Appointement changed to : ' + date);
      callback(results)
    })
  },

  update_journey: (idMeeting, idUser, data, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_SET(data, '$.user_table[?].journey', CAST(? AS JSON)) WHERE idMeeting = ?"

    db.query(query, [idUser, data, idMeeting], (error, results, fields) => {
      if (error) throw error;

      console.log('Journey has been updated for user: ', idUser, 'in', idMeeting);
      callback(results)
    })
  },




  // ----- REMOVE -----

  remove_user: (idMeeting, idUser, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_REMOVE(data, '$.user_table[?]') WHERE idMeeting = ?"

    db.query(query, [idUser, idMeeting], (error, results, fields) => {
      if (error) throw error;

      console.log('User removed : ' + idUser + ' in meeting: ' + idMeeting);
      callback(results)
    })
  },



  // ----- OTHERS -----

  reset_meeting: (idMeeting, callback) => {
    let query = "UPDATE t_meeting SET data = JSON_SET(data, '$.percentage', 0, '$.location', '', '$.date', CAST('{}' AS JSON)) WHERE idMeeting = ?"

    db.query(query, idMeeting, (error, results, fields) => {
      if (error) throw error;

      callback()
    })
  },

  count_users: (idMeeting, callback) => {
    get_users(idMeeting, function(users){

      callback(users.length)
    })
  },

  check_id: (idMeeting, callback) => {
    let query = "SELECT idMeeting FROM t_meeting WHERE idMeeting = ?"

    db.query(query, idMeeting, (error, results, fields) => {
      if (error) throw error;
      var resp = false

      if(results.length != 0){
        if(results[0]['idMeeting'] == idMeeting){
          resp = true
          console.log("Meeting id ", idMeeting, " has been loaded.");
        }
      }

      callback(resp)
    })
  }

}

function get_users(idMeeting, callback){
  let query = "SELECT data->'$.user_table' as user_table FROM t_meeting WHERE idMeeting = ?";

  db.query(query, idMeeting, (error, results, fields) => {
    if (error) throw error;

    var json = JSON.parse(results[0]['user_table'])


    callback(json)
  })
}
