"use strict"

const nodemailer = require('nodemailer')
const credentials = require('../../config/credentials.json')
const config = require('../../config/config.json')
const db = require('../db/db_request')

module.exports = {
  new_meeting: (to, idMeeting) => {
    let sub = "MEETUS: Nouveau Meeting"

    var msg = `
    <b>Vous venez de créer un rendez-vous Meetus</b>

    <p>C'est le moment d'ajouter de nouvelle peronnes à ce rendez-vous.
    Pour ce faire il suffit de partager le code suivant.</p>

    <h4><b>${idMeeting}</b></h4>

    <p>Pour rejoindre le rendez-vous il suffit d'entrer le code sur la page d'accueil du site.</p>

    <p>Une fois sur la page du rendez-vous il suffit d'appuyez sur le bouton "ajouter un membre" et
    d'entrer les informations.</p>

    <p>Cordialement</p>

    <p>L'équipe technique de Meetus</p>
    `


    send(to, sub, msg).catch(console.error);
  },

  missing_people: (idMeeting, date, perc, nbPeople) => {
    let sub = "MEETUS: Rendez-vous incertain"

    var msg = `
    <p>Le rendez-vous <b>${idMeeting}</b> vient d'être programmé et un certain nombre de personne ne pourra être présente.</p>

    <p>La date du <b>${date}</b> vient d'être choisit car elle permet à un maximum de personne d'être présente.</p>

    <p>Malheureusement seul <b>${perc}%</b> des ${nbPeople} membres pourront être présent ce jour-ci.</p>

    <p>Ce mail a pour but d'être informatif et vous demande de prendre note de cette information.</p>

    <p>Cordialement</p>

    <p>L'équipe technique de Meetus</p>
    `

    db.get_emails(idMeeting, function(emails){

      emails.forEach(function(e){
        send(e, sub, msg).catch(console.error);
      })
    })

  },

  new_member: (idMeeting, data) => {
    let sub = "MEETUS: Bien arrivé"

    var msg = `
    <p>Vous venez d'ajouter un compte au rendez-vous <b>${idMeeting}</b>.</p>

    <p>Les informations du compte</p>

    <p><b>Nom</b>   ${data.name}</p>
    <p><b>Email</b> ${data.email}</p>
    <p><b>Ville</b> ${data.address}</p>

    <b>Calendar</b>
    <ul>
    ${data.calendar.map(date => `<li>${date.day}/${date.month}/${date.year} from: ${date.from.hour}:${date.from.minutes} to: ${date.to.hour}:${date.to.minutes}</li>`)}
    </ul>


    <p>Cordialement</p>

    <p>L'équipe technique de Meetus</p>
    `


    send(data.email, sub, msg).catch(console.error);
  }

}

// async..await is not allowed in global scope, must use a wrapper
async function send(to, sub, msg){

  // Generate test SMTP service account from ethereal.email
  // Only needed if you don't have a real mail account for testing
  //let testAccount = await nodemailer.createTestAccount();

  // create reusable transporter object using the default SMTP transport
  let transporter = nodemailer.createTransport({
    host: config.email.server,
    port: config.email.port,
    secure: config.email.secure, // true for 465, false for other ports
    auth: {
      user: credentials.gmail.account, // generated ethereal user
      pass: credentials.gmail.password // generated ethereal password
    }
  });

  // send mail with defined transport object
  let info = await transporter.sendMail({
    from: '"Marc Vachon" <' + credentials.gmail.account + '>', // sender address
    to: to, // list of receivers
    subject: sub, // Subject line
    html: msg
  });

  console.log("Email sent to:", to)
}
