"use strict"
var http = require('../../api/http_request')
var db = require('../db/db_request')

module.exports = {
  update_journeys: (idMeeting, callback) => {
    db.get_meeting(idMeeting, function(data){
      var city_to = data.location

      var date = {day: data.date.year + "-" + data.date.month + "-" + data.date.day, time:  data.date.time.hour + "%3A" + data.date.time.minutes }

      var promises = []

      for (var i = 0; i < data.user_table.length; i++) {
        var journey = set_path(idMeeting, i, data.user_table[i].address, city_to, date)

        promises.push(journey)
      }

      Promise.all(promises).then(function(){
        callback("Updated")
      })

    })
  },

  update_center: (idMeeting, callback) => {

    // faire une request avec toutes les infos et séparer en ce qu'on a besoin
    db.get_meeting(idMeeting, function(data){
      console.log("ALGO: locations received");

      var date = {day: data.date.year + "-" + data.date.month + "-" + data.date.day, time:  data.date.time.hour + "%3A" + data.date.time.minutes }

      var tableLoc = []

      for (var i = 0; i < data.user_table.length; i++) {
        tableLoc.push({lat: data.user_table[i].coordinate.lat, lon: data.user_table[i].coordinate.lon})
      }

      var center = find_geo_center(tableLoc)
      console.log("ALGO: center received: lat " + center.lat + ", lon " + center.lon);

      choose_city(idMeeting, center, date, function(city){
        console.log("ALGO: city chosen: " + city);

        // Si la ville est différente
        appointement_compare(idMeeting, date, city, function(same){
          if (!same) {

            db.update_city_to(idMeeting, city, function(){
              callback()
            })

          }

        })

      })
    })
  }
}


function set_path(idMeeting, idUser, city_from, city_to, date){
  return new Promise(function(resolve){

    var path = http.get_path(city_from, city_to, date)

    path.then(function(data){
      console.log(data);
      db.update_journey(idMeeting, idUser, JSON.stringify(data), function(){
        resolve('ok')
      })

    })

  })
}

function find_geo_center(tableLoc){
  var x = 0, y = 0;

  tableLoc.forEach(function(val){
    x += val.lon
    y += val.lat
  })

  x /= tableLoc.length
  y /= tableLoc.length

  return {lat: y, lon: x}
}

/**
* [choose_city description]
* @param  {[type]}   idMeeting [description]
* @param  {[type]}   center    [description]
* @param  {[type]}   date      [description]
* @param  {Function} callback  [description]
* @return {[type]}             [description]
*/
function choose_city(idMeeting, center, date, callback){
  var cities, city, table_cities = []

  var end = false
  //  while (!end) {

  var cities = http.get_cities(center)


  cities.then(function(result){

    console.log("Cities received : ", result);
    var promises = []

    for (var i = 0; i < result.length; i++) {
      city = result[i]

      var validation = validate_city(idMeeting, result[i], date)

      promises.push(validation)

      validation.then(function(data){

        console.log("Data city :", data);

        //if(data.diff != "-Infinity"){
        table_cities.push(data)
        //}
      })
    }


    Promise.all(promises).then(function(){

      // take the shortest path
      if(table_cities.length != 0){

        city = table_cities[0]

        for (var i = 0; i < table_cities.length; i++) {

          if(table_cities[i].diff < city.diff){
            city = table_cities[i]
          }
        }
      } else {
        console.log("There's no city available");
      }



      callback(city.city)
    })

  }, function(error){
    console.log(error);

  })

  //console.log("valeur i:" + i);

  //center = choose_new_point(center)

  //}

}

function choose_new_point(point){
  // approximately
  var max_lat = 10.455201,
  max_lon = 47.793944,
  min_lat = 5.975070,
  min_lon = 45.833100

  return point
}

function validate_city(idMeeting, city_to, date){
  var maxDiff = 60, diff = 0, tableTimes = [], time, sum = 0, avg = 0


  return new Promise(function(res, rej){

    db.get_cities(idMeeting, function(tableCities){

      var promises = [];

      for (var i = 0; i < tableCities.length; i++) {
        // path is a promise
        var journey = http.get_path_duration(tableCities[i], city_to, date)
        // push all promise in table
        promises.push(journey)

        // wait for result
        journey.then(function(res){

          if(res['connections'].length > 0){

            // convert time in min
            times_to_min(res['connections'][0]['duration'], function(minutes){
              tableTimes.push(minutes)

            })
          }
        })
      }


      // wait for all promises to be done
      Promise.all(promises).then(function(){
        if (tableTimes.length > 1) {

          var min = Math.min(...tableTimes)
          var max = Math.max(...tableTimes)

          if(max != min){
            diff = max - min
          }else{
            diff = min
          }

          sum = tableTimes.reduce((previous, current) => current += previous);
          avg = parseInt(sum / tableTimes.length);
        }

        res({city: city_to, diff: diff, avg: avg});

      }, function(error){
        console.log(error);
      })

    })

  })
}

function appointement_compare(idMeeting, date, city, callback){
  callback(false)
}

function times_to_min(time, callback){
  var t = time.split('d')[1].split(':')
  var result = parseInt(t[0]) * 60 + parseInt(t[1])
  callback(result)
}
